/*!
\file gps.c
\brief Kommunikation zwischen AVR und Navilock GPS Modul
\author Martin Matysiak (mail@k621.de)
\author V. Pippan (webmaster@vpippan.at)
\date 2013-10-18
\version 20131018

Bei Fragen und Verbesserungen wendet euch per EMail an mich.

Datenblätter zum Umgang mit Navilock GPS Modulen:

<a href="http://www.navilock.de/download/Dokumente_SLASH_Sonstiges/60415_-_Datenblatt_u-blox_GPS_Module/481">Datenblatt Navilock U-Blox</a><br>
<a href="http://www.sparkfun.com/datasheets/GPS/NMEA%20Reference%20Manual1.pdf">NMEA Reference Manual</a>

Copyright (c) 2008 Martin Matysiak<br>
Copyright 2008-2013 V. Pippan (webmaster@vpippan.at)

This file is part of ATmega8 GPS Logger.

ATmega8 GPS Logger is free software: you can redistribute it and/or modify<br>
it under the terms of the GNU General Public License as published by<br>
the Free Software Foundation, either version 3 of the License, or any later version.<br>

ATmega8 GPS Logger is distributed in the hope that it will be useful,<br>
but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the<br>
GNU General Public License for more details.<br>

You should have received a copy of the GNU General Public License<br>
along with ATtiny2313 Cessna Lichtsteuerung.  If not, see<br>
<a href="http://www.gnu.org/licenses" target="blank">www.gnu.org/licenses</a>.

*/

#include "gps.h"

/*! \brief Routine zur Initialisierung des GPS Moduls
 *
 * */
void gps_init()
{
	// USART Schnittstelle initalisieren
	uart_init(UART_BAUD_SELECT(GPS_BAUD, F_CPU));

	// Interrupts global einschalten
	sei();
}

/*! \brief Gibt sobald verfügbar ein Zeichen vom UART zurück
 *
 * */
char gps_get_char()
{
	unsigned int in = 0;

	do
	{
		in = uart_getc();
	}
	while((in & UART_NO_DATA) || (in & UART_BUFFER_OVERFLOW));

	return((char) in);
}

/*! \brief Empfängt einen vollständige NMEA-Befehl und gibt diesen dann zurück
 *
 * */
void gps_get_nmea(char *buf, uint8_t bufSize)
{
	char inChar;
	uint8_t curSize = 0;

	// warten bis das 1. zeichen ($) des nmea strings kommt
	do
	{
		inChar = gps_get_char();
	}
	while(inChar != '$');

	// nachdem das 1.zeichen erkannt wurde den string einlesen
	while (inChar != '\n' && curSize < bufSize - 2)
	{
		*buf++ = inChar;
		curSize++;
		inChar = gps_get_char();
	}

	*buf++ = '\n';
	*buf = '\0';
}
